unit RasterFontEditor;
(*******************************************************************************
*                       RasterFontEditor component                             *
*                                  by                                          *
*                            Vaclav Kourda                                     *
*                                                                              *
*                                 1998                                         *
*                                                                              *
* With this component you can easily manipulate position, width, height, colour*
* of rasterfont characters in a text                                           *
*******************************************************************************)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

const
  MAX_SIZE = 100;

type
  TCharItem = record
                Top, Left : Single;
                Width, Height : Single;
                Color : TColor;
                Character : char;
              end;

  TBitMapFont = record
                  FontFile : string[255];
                  FontHeight, FontWidth : integer;
                  CharSetWidth, CharSetHeight : integer;
                  LineSize : integer;
                end;

  TTextArray = array [1..MAX_SIZE, 1..MAX_SIZE] of TCharItem;

  TItem = record
            Row : integer;
            Col : integer;
          end;
  PItem = ^TItem;

  TRasterFontEditor = class(TImage)
  private
    { Private declarations }
    bSpaceSelect : Boolean;
    cSelectedColor : TColor;
    TextA : TTextArray;
    ArrayHeight, ArrayWidth : Word;
    bFixHorPos, bFixVerPos : boolean;
    bFixWidth, bFixHeight : boolean;

    bValidValues : boolean;
    bmfBitMapFont : TBitMapFont;
    iSpaceChars, iSpaceRows : integer;
    bSelected : boolean;
    bMultiSelect : boolean;
    SelecTop, SelecLeft, SelecWidth, SelecHeight : integer;
    PrevX, PrevY : integer;
    bmTemp : TBitMap;
    bmFont : TBitMap;
    lSelChars : TList;
    procedure DeleteItemsFromList;
    procedure DrawBorderBox (XPos, YPos, XWidth, YHeight : integer);
    function SelectChar( const X,Y : integer;
                         var XPos, YPos, XWidth, YHeight : integer) : boolean;
    function SelectChars( const StartX,StartY,EndX,EndY : integer) : boolean;
    procedure RebuildArea(const bExamine : boolean);
    function InPosList(const Row,Col : integer) : boolean;
    procedure GetOriginalRasterFont(const Row,Col : integer; const C : TCanvas; const Colour : TColor);
    procedure SetBitMapFont(Value : TBitMapFont);
    procedure SetSpaceSelect(Value : Boolean);
    procedure SetSelectedColor(Value : TColor);
    procedure SetTextArrayHeight(Value : Word);
    procedure SetTextArrayWidth(Value : Word);
    procedure SetSpaceChars(Value : Integer);
    procedure SetSpaceRows(Value : Integer);
  protected
    { Protected declarations }
    procedure SetWidth;
    procedure SetHeight;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
  public
    { Public declarations }
    bmPicture : TBitMap;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure SetArrayItem( const Row, Col : integer; Value : TCharItem; const ReDraw : boolean);
    function GetArrayItem(const Row,Col : integer) : TCharItem;
    procedure Clear;
    procedure DrawTextArray;
  published
    { Published declarations }
    property SpaceSelect : Boolean read bSpaceSelect write SetSpaceSelect default False;
    property SelectedColor : TColor read cSelectedColor write SetSelectedColor default clBlack;
    property TextArrayHeight : Word read ArrayHeight write SetTextArrayHeight default 20;
    property TextArrayWidth : Word read ArrayWidth write SetTextArrayWidth default 20;
    property BitMapFont : TBitMapFont read bmfBitMapFont write SetBitMapFont;
    property SpaceChars : Integer read iSpaceChars write SetSpaceChars default 1;
    property SpaceRows : Integer read iSpaceRows write SetSpaceRows default 1;
    property FixHorPos : Boolean read bFixHorPos write bFixHorPos default True;
    property FixVerPos : Boolean read bFixVerPos write bFixVerPos default True;
    property FixWidth : Boolean read bFixWidth write bFixWidth default True;
    property FixHeight : Boolean read bFixHeight write bFixHeight default True;
    property SelectLeft : integer read SelecLeft;
    property SelectTop : integer read SelecTop;
    property SelectWidth : integer read SelecWidth;
    property SelectHeight : integer read SelecHeight;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TRasterFontEditor]);
end;

constructor TRasterFontEditor.Create(AOwner: TComponent);
var i,j : integer;
begin
  inherited Create(AOwner);
  Height := 105;
  Width := 105;

  bmTemp := TBitMap.Create;
  bmTemp.TransparentColor := clWhite;
  bmTemp.TransparentMode := tmFixed;
  bmTemp.Transparent := True;

  bmPicture := TBitMap.Create;
  bmPicture.TransparentColor := clWhite;
  bmPicture.TransparentMode := tmFixed;
  bmPicture.Transparent := False;
  bmPicture.Canvas.Brush.Color := clWhite;

  bmFont := TBitMap.Create;

  Canvas.Pen.Width := 1;
  Canvas.Pen.Color := clBlack;
  lSelChars := TList.Create;
  bSelected := False;
  bMultiSelect := False;
  bValidValues := False;

  ArrayWidth := 20;
  ArrayHeight := 20;
  iSpaceChars := 1;
  iSpaceRows := 1;

  SelecLeft := 0;
  SelecTop := 0;
  SelecWidth := 0;
  SelecHeight := 0;

  for i:=1 to MAX_SIZE do
    for j:=1 to MAX_SIZE do
    begin
      TextA[i,j].Top := 0;
      TextA[i,j].Left := 0;
      TextA[i,j].Width := 0;
      TextA[i,j].Height := 0;
      TextA[i,j].Color := clBlack;
      TextA[i,j].Character := ' ';
    end;
end;

destructor TRasterFontEditor.Destroy;
begin
  bmFont.Free;
  bmTemp.Free;
  bmPicture.Free;
  DeleteItemsFromList;
  lSelChars.Free;
  inherited Destroy;
end;

procedure TRasterFontEditor.DeleteItemsFromList;
var i : integer;
    pCharPos : PItem;
begin
  for i:=0 to lSelChars.Count-1 do
  begin
    pCharPos := lSelChars.Items[i];
    Dispose(pCharPos);
  end;
  lSelChars.Clear;
end;

procedure TRasterFontEditor.DrawBorderBox (XPos, YPos, XWidth, YHeight : integer);
begin
  Canvas.Draw(0,0,bmPicture);
  Canvas.MoveTo(XPos,YPos);
  Canvas.LineTo(XPos+XWidth,YPos);
  Canvas.LineTo(XPos+XWidth,YPos+YHeight);
  Canvas.LineTo(XPos,YPos+YHeight);
  Canvas.LineTo(XPos,YPos);
  Update;
end;

function TRasterFontEditor.SelectChar( const X,Y : integer; var XPos, YPos, XWidth, YHeight : integer) : boolean;
var i,j : integer;
    pCharPos : PItem;
begin
  Result := False;
  DeleteItemsFromList;
  for i:=1 to ArrayHeight do
  begin
    for j:=1 to ArrayWidth do
    begin
      if (TextA[i,j].Top <= Y) and
         (TextA[i,j].Left <= X) and
         (TextA[i,j].Left+TextA[i,j].Width >=X) and
         (TextA[i,j].Top+TextA[i,j].Height >=Y) then
      begin
        if bSpaceSelect or (TextA[i,j].Character<>' ') then
        begin
          New(pCharPos);
          pCharPos^.Row := i;
          pCharPos^.Col := j;
          lSelChars.Add(pCharPos);

          Result := True;
          cSelectedColor := TextA[i,j].Color;
          XPos := Round(TextA[i,j].Left);
          YPos := Round(TextA[i,j].Top);
          XWidth := Round(TextA[i,j].Width);
          YHeight := Round(TextA[i,j].Height);
        end;
      end;
      if Result then
        break;
    end;
    if Result then
      break;
  end;
end;

function TRasterFontEditor.SelectChars( const StartX,StartY,EndX,EndY : integer) : boolean;
var i,j : integer;
    pCharPos : PItem;
    MinX, MaxX, MinY, MaxY : integer;
    SelecRight, SelecBottom : integer;
begin
  Result := False;
  DeleteItemsFromList;

  if StartX<=EndX then
  begin
    MinX := StartX;
    MaxX := EndX;
  end
  else
  begin
    MinX := EndX;
    MaxX := StartX;
  end;

  if StartY<=EndY then
  begin
    MinY := StartY;
    MaxY := EndY;
  end
  else
  begin
    MinY := EndY;
    MaxY := StartY;
  end;

  for i:=1 to ArrayHeight do
  begin
    for j:=1 to ArrayWidth do
    begin
      if (TextA[i,j].Top >= MinY) and
         (TextA[i,j].Left >= MinX) and
         (TextA[i,j].Left+TextA[i,j].Width <= MaxX) and
         (TextA[i,j].Top+TextA[i,j].Height <= MaxY) then
      begin
        if bSpaceSelect or (TextA[i,j].Character<>' ') then
        begin
          New(pCharPos);
          pCharPos^.Row := i;
          pCharPos^.Col := j;
          lSelChars.Add(pCharPos);

          Result := True;
          cSelectedColor := TextA[i,j].Color;
        end;
      end;
    end;
  end;

  if Result then
  begin
    if lSelChars.Count>0 then
    begin
      pCharPos := lSelChars.Items[0];
      SelecLeft := Round(TextA[pCharPos^.Row,pCharPos^.Col].Left);
      SelecTop := Round(TextA[pCharPos^.Row,pCharPos^.Col].Top);
      SelecRight := Round( TextA[pCharPos^.Row,pCharPos^.Col].Left +
                           TextA[pCharPos^.Row,pCharPos^.Col].Width);
      SelecBottom:= Round( TextA[pCharPos^.Row,pCharPos^.Col].Top +
                           TextA[pCharPos^.Row,pCharPos^.Col].Height);
    end;

    for i:=0 to lSelChars.Count-1 do
    begin
      pCharPos := lSelChars.Items[i];
      if Round(TextA[pCharPos^.Row,pCharPos^.Col].Left)< SelecLeft then
        SelecLeft := Round(TextA[pCharPos^.Row,pCharPos^.Col].Left);
      if Round(TextA[pCharPos^.Row,pCharPos^.Col].Top)< SelecTop then
        SelecTop := Round(TextA[pCharPos^.Row,pCharPos^.Col].Top);

      if Round( TextA[pCharPos^.Row,pCharPos^.Col].Left+
                TextA[pCharPos^.Row,pCharPos^.Col].Width)> SelecRight then
        SelecRight := Round( TextA[pCharPos^.Row,pCharPos^.Col].Left +
                             TextA[pCharPos^.Row,pCharPos^.Col].Width);
      if Round( TextA[pCharPos^.Row,pCharPos^.Col].Top+
                TextA[pCharPos^.Row,pCharPos^.Col].Height)> SelecBottom then
        SelecBottom := Round( TextA[pCharPos^.Row,pCharPos^.Col].Top +
                              TextA[pCharPos^.Row,pCharPos^.Col].Height);
    end;
    SelecWidth := SelecRight - SelecLeft;
    SelecHeight := SelecBottom - SelecTop;
  end;
end;

function TRasterFontEditor.InPosList(const Row,Col : integer) : boolean;
var i : integer;
    pCharPos : PItem;
begin
  Result := False;
  for i:=0 to lSelChars.Count-1 do
  begin
    pCharPos := lSelChars.Items[i];
    if (pCharPos^.Row=Row) and (pCharPos^.Col=Col) then
      Result := True;
    if Result then
      break;
  end;
end;

procedure TRasterFontEditor.RebuildArea(const bExamine : boolean);
var i,j : integer;
    A,B,C,D,E,F,G,H : boolean;
begin
  bmPicture.Transparent := True;
  bmPicture.Canvas.FillRect(Rect(SelecLeft,SelecTop,SelecLeft+SelecWidth+1,SelecTop+SelecHeight+1));

  for i:=1 to ArrayHeight do
  begin
    for j:=1 to ArrayWidth do
    begin
      A := (SelecTop<=TextA[i,j].Top) and (TextA[i,j].Top<=SelecTop+SelecHeight);
      B := (TextA[i,j].Left<=SelecLeft+SelecWidth) and (SelecLeft+SelecWidth<=TextA[i,j].Left+TextA[i,j].Width);
      C := (TextA[i,j].Left<=SelecLeft) and (SelecLeft<=TextA[i,j].Left+TextA[i,j].Width);
      D := (SelecTop<=TextA[i,j].Top+TextA[i,j].Height) and (TextA[i,j].Top+TextA[i,j].Height<=SelecTop+SelecHeight);
      E := (SelecLeft<=TextA[i,j].Left) and (TextA[i,j].Left<=SelecLeft+SelecWidth);
      F := (TextA[i,j].Top<=SelecTop+SelecHeight) and (SelecTop+SelecHeight<=TextA[i,j].Top+TextA[i,j].Height);
      G := (TextA[i,j].Top<=SelecTop) and (SelecTop<=TextA[i,j].Top+TextA[i,j].Height);
      H := (SelecLeft<=TextA[i,j].Left+TextA[i,j].Width) and (TextA[i,j].Left+TextA[i,j].Width<=SelecLeft+SelecWidth);
      if (A and (B or C)) or (D and (B or C)) or (E and (F or G)) or (H and (F or G)) then
      begin
        if (not bExamine) or (not InPosList(i,j)) then
        begin
          if bSpaceSelect or (TextA[i,j].Character<>' ') then
          begin
            bmTemp.Width := bmfBitMapFont.FontWidth;
            bmTemp.Height := bmfBitMapFont.FontHeight;
            GetOriginalRasterFont(i,j,bmTemp.Canvas,TextA[i,j].Color);
            bmPicture.Canvas.StretchDraw (Rect(Round(TextA[i,j].Left),Round(TextA[i,j].Top),
                                               Round(TextA[i,j].Left+TextA[i,j].Width),
                                               Round(TextA[i,j].Top+TextA[i,j].Height)),
                                          bmTemp);
          end;
        end
      end;
    end;
  end;
  bmPicture.Transparent := False;;
end;

procedure TRasterFontEditor.GetOriginalRasterFont(const Row,Col : integer; const C : TCanvas; const Colour : TColor);
var cTemp : char;
    iRow, iCol : integer;
    xSource, ySource : integer;
begin
  cTemp := TextA[Row,Col].Character;

  iRow := ord(cTemp) div bmfBitMapFont.CharSetWidth;
  iCol := ord(cTemp) mod bmfBitMapFont.CharSetWidth;

  xSource := bmfBitMapFont.LineSize+iCol*(bmfBitMapFont.FontWidth+bmfBitMapFont.LineSize);
  ySource := bmfBitMapFont.LineSize+iRow*(bmfBitMapFont.FontHeight+bmfBitMapFont.LineSize);

  if Colour<>clBlack then
  begin
    C.Brush.Color := Colour;
    C.BrushCopy (Rect(0,0,bmfBitMapFont.FontWidth,bmfBitMapFont.FontHeight),
                bmFont,
                Rect(xSource,ySource,
                xSource+bmfBitMapFont.FontWidth,
                ySource+bmfBitMapFont.FontHeight),
                clBlack);
  end
  else
  begin
    C.CopyRect( Rect(0,0,bmfBitMapFont.FontWidth,bmfBitMapFont.FontHeight),
                bmFont.Canvas,
                Rect(xSource,ySource,
                xSource+bmfBitMapFont.FontWidth,
                ySource+bmfBitMapFont.FontHeight));
  end;
end;

procedure TRasterFontEditor.SetSpaceSelect(Value : Boolean);
begin
  if Value <> bSpaceSelect then
    bSpaceSelect := Value;
end;

procedure TRasterFontEditor.SetSelectedColor(Value : TColor);
var i : integer;
    pCharPos : PItem;
begin
  if bSelected then
  begin
    for i:=0 to lSelChars.Count-1 do
    begin
      pCharPos := lSelChars.Items[i];
      TextA[pCharPos^.Row,pCharPos^.Col].Color := Value;
    end;

    RebuildArea(False);

    DrawBorderBox (SelecLeft,SelecTop,SelecWidth,SelecHeight);
  end;
end;

procedure TRasterFontEditor.SetTextArrayHeight(Value : Word);
begin
  if Value <> ArrayHeight then
  begin
    ArrayHeight := Value;
    SetHeight;
  end;
end;

procedure TRasterFontEditor.SetTextArrayWidth(Value : Word);
begin
  if Value <> ArrayWidth then
  begin
    ArrayWidth := Value;
    SetWidth;
  end;
end;

procedure TRasterFontEditor.SetBitMapFont(Value : TBitMapFont);
begin
  if FileExists(Value.FontFile) and (Value.FontHeight>0) and (Value.FontWidth>0) and
     (Value.CharSetWidth>0) and (Value.CharSetHeight>0) and (Value.LineSize>0) then
  begin
    bValidValues := True;

    if (bmfBitMapFont.FontHeight<>Value.FontHeight) then
    begin
      bmfBitMapFont.FontHeight := Value.FontHeight;
      SetHeight;
    end;
    if (bmfBitMapFont.FontWidth<>Value.FontWidth) then
    begin
      bmfBitMapFont.FontWidth := Value.FontWidth;
      SetWidth;
    end;
    bmfBitMapFont := Value;
    bmFont.LoadFromFile(bmfBitMapFont.FontFile);
    bmPicture.Canvas.Brush.Color := clWhite;
    bmPicture.Canvas.FillRect(Rect(0,0,bmPicture.Width,bmPicture.Height));
    Picture.Bitmap := bmPicture;
  end;
end;

procedure TRasterFontEditor.SetSpaceChars(Value : Integer);
begin
  if (Value>=0) and (iSpaceChars<>Value) then
  begin
    iSpaceChars := Value;
    SetWidth;
  end;
end;

procedure TRasterFontEditor.SetSpaceRows(Value : Integer);
begin
  if (Value>=0) and (iSpaceRows<>Value) then
  begin
    iSpaceRows := Value;
    SetHeight;
  end;
end;

procedure TRasterFontEditor.SetWidth;
begin
  if bValidValues then
  begin
    Width := (ArrayWidth*bmfBitMapFont.FontWidth)+ ((ArrayWidth-1)* iSpaceChars);
    bmPicture.Width := (ArrayWidth*bmfBitMapFont.FontWidth)+ ((ArrayWidth-1)* iSpaceChars);
  end;
end;

procedure TRasterFontEditor.SetHeight;
begin
  if bValidValues then
  begin
    Height := (ArrayHeight*bmfBitMapFont.FontHeight)+ ((ArrayHeight-1)* iSpaceRows);
    bmPicture.Height := (ArrayHeight*bmfBitMapFont.FontHeight)+ ((ArrayHeight-1)* iSpaceRows);
  end;
end;

procedure TRasterFontEditor.SetArrayItem( const Row, Col : integer; Value : TCharItem;
                                          const ReDraw : boolean);
begin
  if (Row>0) and (Col>0) then
  begin
    TextA[Row,Col]:=Value;
    if ReDraw then
    begin
      SelecTop := Round(Value.Top-1);
      SelecLeft := Round(Value.Left-1);
      SelecWidth := Round(Value.Width+1);
      SelecHeight := Round(Value.Height+1);
      ReBuildArea(False);

      SelecTop := 0;
      SelecLeft := 0;
      SelecWidth := 0;
      SelecHeight := 0;
      Canvas.Draw(0,0,bmPicture);
      Update;
    end;
  end;
end;

function TRasterFontEditor.GetArrayItem(const Row,Col : integer) : TCharItem;
begin
  if (Row>0) and (Col>0) then
    Result := TextA[Row,Col];
end;

procedure TRasterFontEditor.MouseDown( Button: TMouseButton; Shift: TShiftState;
                     X, Y: Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);

  if Screen.Cursor=crDefault then
  begin
    if SelectChar(X,Y,SelecLeft,SelecTop,SelecWidth,SelecHeight) then
    begin
      bSelected := True;
      bMultiSelect := False;
      DrawBorderBox (SelecLeft,SelecTop,SelecWidth,SelecHeight);

      if (X=SelecWidth+SelecLeft) or (X=SelecLeft) then
      begin
        Screen.Cursor := crSizeWE;
      end
      else if (Y=SelecHeight+SelecTop) or (Y=SelecTop) then
      begin
        Screen.Cursor := crSizeNS;
      end
      else
      begin
        Screen.Cursor := crDrag;
      end
    end
    else
    begin
      Screen.Cursor := crDefault;
      bSelected := False;
      Canvas.Draw(0,0,bmPicture);
      Update;
    end;
  end;

  PrevX := X;
  PrevY := Y;
end;

procedure TRasterFontEditor.MouseMove( Shift: TShiftState; X, Y: Integer);
var XDiff, YDiff, i : integer;
    pCharPos : PItem;
    StretchX, StretchY : Extended;
begin
  inherited MouseMove( Shift, X, Y);

  if not ((ssLeft) in Shift) then
  begin
    if (bSelected) then
    begin
    {Set the shape of the cursor}
      if (X=SelecWidth+SelecLeft) then
      begin
        Screen.Cursor := crSizeWE;
      end
      else if (Y=SelecHeight+SelecTop) then
      begin
        Screen.Cursor := crSizeNS;
      end
      else if (X<SelecWidth+SelecLeft) and (X>=SelecLeft) and
              (Y<SelecHeight+SelecTop) and (Y>=SelecTop) then
      begin
        Screen.Cursor := crDrag;
      end
      else
        Screen.Cursor := crDefault;
    {END OF Set the shape of the cursor}
    end
    else
      Screen.Cursor := crDefault;
  end;

  if ((ssLeft) in Shift) and (Screen.Cursor=crDrag) then
  begin
    if (X<>PrevX) or (Y<>PrevY) then
    begin
      XDiff := X-PrevX;
      YDiff := Y-PrevY;

      if bFixHorPos then
        XDiff := 0;
      if bFixVerPos then
        YDiff := 0;

      RebuildArea(True);

      Inc(SelecLeft,XDiff);
      Inc(SelecTop,YDiff);

      for i:=0 to lSelChars.Count-1 do
      begin
        pCharPos := lSelChars.Items[i];
        bmTemp.Width := bmfBitMapFont.FontWidth;
        bmTemp.Height := bmfBitMapFont.FontHeight;
        GetOriginalRasterFont(pCharPos^.Row,pCharPos^.Col,bmTemp.Canvas,TextA[pCharPos^.Row,pCharPos^.Col].Color);

        TextA[pCharPos^.Row,pCharPos^.Col].Top := TextA[pCharPos^.Row,pCharPos^.Col].Top + YDiff;
        TextA[pCharPos^.Row,pCharPos^.Col].Left := TextA[pCharPos^.Row,pCharPos^.Col].Left + XDiff;

        bmPicture.Canvas.StretchDraw ( Rect( Round(TextA[pCharPos^.Row,pCharPos^.Col].Left),Round(TextA[pCharPos^.Row,pCharPos^.Col].Top),
                                             Round(TextA[pCharPos^.Row,pCharPos^.Col].Left+TextA[pCharPos^.Row,pCharPos^.Col].Width),
                                             Round(TextA[pCharPos^.Row,pCharPos^.Col].Top+TextA[pCharPos^.Row,pCharPos^.Col].Height)),
                                       bmTemp);
      end;

      PrevX := X;
      PrevY := Y;
      DrawBorderBox (SelecLeft,SelecTop,SelecWidth,SelecHeight);
      Update;
    end;
  end
  else
  if ((ssLeft) in Shift) and ((Screen.Cursor=crSizeNS) or (Screen.Cursor=crSizeWE)) then
  begin
    if (X<>PrevX) or (Y<>PrevY) then
    begin
      XDiff := X-PrevX;
      YDiff := Y-PrevY;

      if bFixWidth then
        XDiff := 0;
      if bFixHeight then
        YDiff := 0;

      if (SelecWidth+XDiff>0) and (SelecHeight+YDiff>0) then
      begin
        RebuildArea(True);

        StretchX := 1+(XDiff/SelecWidth);
        StretchY := 1+(YDiff/SelecHeight);

        Inc(SelecWidth,XDiff);
        Inc(SelecHeight,YDiff);

        for i:=0 to lSelChars.Count-1 do
        begin
          pCharPos := lSelChars.Items[i];
          bmTemp.Width := bmfBitMapFont.FontWidth;
          bmTemp.Height := bmfBitMapFont.FontHeight;
          GetOriginalRasterFont(pCharPos^.Row,pCharPos^.Col,bmTemp.Canvas,TextA[pCharPos^.Row,pCharPos^.Col].Color);

          TextA[pCharPos^.Row,pCharPos^.Col].Top := SelecTop+((TextA[pCharPos^.Row,pCharPos^.Col].Top-SelecTop) * StretchY);
          TextA[pCharPos^.Row,pCharPos^.Col].Left := SelecLeft+((TextA[pCharPos^.Row,pCharPos^.Col].Left-SelecLeft) * StretchX);
          TextA[pCharPos^.Row,pCharPos^.Col].Width := TextA[pCharPos^.Row,pCharPos^.Col].Width * StretchX;
          TextA[pCharPos^.Row,pCharPos^.Col].Height := TextA[pCharPos^.Row,pCharPos^.Col].Height * StretchY;

          bmPicture.Canvas.StretchDraw ( Rect( Round(TextA[pCharPos^.Row,pCharPos^.Col].Left),Round(TextA[pCharPos^.Row,pCharPos^.Col].Top),
                                               Round(TextA[pCharPos^.Row,pCharPos^.Col].Left+TextA[pCharPos^.Row,pCharPos^.Col].Width),
                                               Round(TextA[pCharPos^.Row,pCharPos^.Col].Top+TextA[pCharPos^.Row,pCharPos^.Col].Height)),
                                         bmTemp);
        end;

        PrevX := X;
        PrevY := Y;
        DrawBorderBox (SelecLeft,SelecTop,SelecWidth,SelecHeight);
        Update;
      end;
    end;
  end
  else
  if ((ssLeft) in Shift) then
  begin
    Canvas.Draw(0,0,bmPicture);
    Canvas.Pen.Style := psDash;
    Canvas.MoveTo(PrevX,PrevY);
    Canvas.LineTo(X,PrevY);
    Canvas.LineTo(X,Y);
    Canvas.LineTo(PrevX,Y);
    Canvas.LineTo(PrevX,PrevY);
    Update;
    Canvas.Pen.Style := psSolid;
    bMultiSelect:=True;
  end;
end;

procedure TRasterFontEditor.MouseUp( Button: TMouseButton; Shift: TShiftState;
                   X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);

  if (Screen.Cursor=crDefault) and bMultiSelect then
  begin
    if SelectChars(PrevX,PrevY,X,Y) then
    begin
      bSelected := True;
      DrawBorderBox (SelecLeft,SelecTop,SelecWidth,SelecHeight);
      bMultiSelect := False;
    end
    else
    begin
      Screen.Cursor := crDefault;
      bSelected := False;
      Canvas.Draw(0,0,bmPicture);
      Update;
    end;
  end;

end;

procedure TRasterFontEditor.Clear;
var i,j : integer;
begin
  for i:=1 to ArrayHeight do
    for j:=1 to ArrayWidth do
    begin
      TextA[i,j].Top := 0;
      TextA[i,j].Left := 0;
      TextA[i,j].Width := 0;
      TextA[i,j].Height := 0;
      TextA[i,j].Color := clBlack;
      TextA[i,j].Character := ' ';
    end;
  Canvas.Brush.Color := clWhite;
  Canvas.FillRect(Rect(0,0,Width,Height));
  bmPicture.Canvas.Brush.Color := clWhite;
  bmPicture.Canvas.FillRect(Rect(0,0,bmPicture.Width,bmPicture.Height));
  Picture.Bitmap := bmPicture;
end;

procedure TRasterFontEditor.DrawTextArray;
var i,j : integer;
begin
  for i:=1 to ArrayHeight do
    for j:=1 to ArrayWidth do
      begin
        bmTemp.Width := bmfBitMapFont.FontWidth;
        bmTemp.Height := bmfBitMapFont.FontHeight;
        GetOriginalRasterFont(i,j,bmTemp.Canvas,TextA[i,j].Color);
        bmPicture.Canvas.StretchDraw ( Rect( Round(TextA[i,j].Left),Round(TextA[i,j].Top),
                                               Round(TextA[i,j].Left+TextA[i,j].Width),
                                               Round(TextA[i,j].Top+TextA[i,j].Height)),
                                       bmTemp);
      end;
  Picture.BitMap := bmPicture;
  Update;
end;

end.
